@@ -12,8 +12,16 @@ o Boolean parameters just need to be present to be set on:
     - colorBars, optimizePDF, cropMarks, pageInformationMarks,
         registrationMarks,
 
+
 o String parameters:
 
+    You can call a predifined preset (.joboptions file) with
+    pdfExportPresetName:
+
+    - pdfExportPresetName: [Press Quality], ...
+
+    Or the the following parameters:
+
     - acrobatCompatibility: 4, 5, 6, 7 or 8
     - colorSpace: CMYK, iGry, rCMY, rRGB, cRGB or unFc
     - colorProfile (output > color > destination) and
