@@ -56,7 +56,8 @@ class IDMLXMLFile(object):
         self._dom = None
 
     def __repr__(self):
-        return "%s %s" % (self.__class__.__name__, self.name)
+        return "<%s object %s at %s>" % (self.__class__.__name__,
+                                         self.name, hex(id(self)))
 
     @property
     def fobj(self):
@@ -367,10 +368,8 @@ class Story(IDMLXMLFile):
     def set_element_content(self, element_id, content):
         self.clear_element_content(element_id)
         element = self.get_element_by_id(element_id)
-        try:
-            self.get_element_content_nodes(element)[0].text = content
-        except IndexError:
-            pass
+        xml_element = XMLElement(element=element)
+        xml_element.set_content(content)
 
     def clear_element_content(self, element_id):
         element = self.get_element_by_id(element_id)
@@ -383,18 +382,19 @@ class Story(IDMLXMLFile):
             content_node.text = ""
 
     def get_element_content_nodes(self, element):
-        return element.xpath(("./ParagraphStyleRange/CharacterStyleRange/Content | \
-                               ./CharacterStyleRange/Content | \
-                               ./XMLElement/CharacterStyleRange/Content | \
-                               ./Content"))
+        return element.xpath(("./ParagraphStyleRange/CharacterStyleRange/Content | "
+                              "./CharacterStyleRange/Content | "
+                              "./XMLElement/CharacterStyleRange/Content | "
+                              "./Content"))
 
     def get_element_content_and_xmlelement_nodes(self, element):
-        return element.xpath(("./ParagraphStyleRange/CharacterStyleRange/Content | \
-                               ./CharacterStyleRange/Content | \
-                               ./CharacterStyleRange/XMLElement | \
-                               ./ParagraphStyleRange/XMLElement | \
-                               ./XMLElement | \
-                               ./Content"))
+        return element.xpath(("./ParagraphStyleRange/CharacterStyleRange/Content | "
+                              "./CharacterStyleRange/Content | "
+                              "./ParagraphStyleRange/CharacterStyleRange/XMLElement | "
+                              "./CharacterStyleRange/XMLElement | "
+                              "./ParagraphStyleRange/XMLElement | "
+                              "./XMLElement | "
+                              "./Content"))
 
     def set_element_id(self, element):
         ref_element = [e for e in element.itersiblings(tag="XMLElement", preceding=True)]
@@ -514,6 +514,13 @@ class Designmap(IDMLXMLFile):
                 etree.Element("{%s}Spread" % IdPkgNS, src=spread.name)
             )
 
+    def prefix(self, prefix):
+        self.prefix_active_layer(prefix)
+        self.prefix_page_start(prefix)
+
+    def prefix_active_layer(self, prefix):
+        self.active_layer = "%s%s" % (prefix, self.active_layer)
+
     def prefix_page_start(self, prefix):
         section_node = self.section_node
         current_page_start = section_node.get(self.page_start_attr)
@@ -568,6 +575,10 @@ class Designmap(IDMLXMLFile):
         layer_node = self.dom.xpath(".//Layer[@Name='%s']" % layer_name)[0]
         return layer_node.get("Self")
 
+    def get_active_layer_name(self):
+        layer_node = self.dom.xpath(".//Layer[@Self='%s']" % self.active_layer)[0]
+        return layer_node.get("Name")
+
 
 class Style(IDMLXMLFile):
     name = "Resources/Styles.xml"
@@ -819,6 +830,27 @@ class XMLElement(Proxy):
         style_range_node.append(content_element)
         self.element.append(style_range_node)
 
+    def set_content(self, content):
+        try:
+            self.get_element_content_nodes()[0].text = content
+        except IndexError:
+            return
+        # Ticket #8 - Fix the style locally.
+        local_style = self.get_local_character_style_range()
+        if local_style is None:
+            super_style = self.get_super_character_style_range()
+            if super_style is not None:
+                # Force super style locally.
+                local_style = self._create_style_range_from_parent(self)
+                self.apply_style_locally(local_style)
+
+    def apply_style_locally(self, style_range_node):
+        """The content node of self is moved into style_range_node. """
+        content_node = self.get_element_content_nodes()[0]
+        style_range_node.append(content_node)
+        self.append(style_range_node)
+
+    # TODO: Why not calling this method directly on parent (which is a XMLElement too)?
     def _create_style_range_from_parent(self, parent):
         parent_style_node = parent.get_character_style_range()
         applied_style = parent_style_node.get("AppliedCharacterStyle")
@@ -864,14 +896,31 @@ class XMLElement(Proxy):
 
     def get_character_style_range(self):
         """The applied style may be contained or the container. """
+        node = self.get_local_character_style_range()
+        if node is None:
+            node = self.get_super_character_style_range()
+        return node
+
+    def get_local_character_style_range(self):
         try:
             node = self.xpath(("./ParagraphStyleRange/CharacterStyleRange | ./CharacterStyleRange"))[0]
         except (IndexError, AttributeError):
-            node = self.getparent()
-            if node.tag != "CharacterStyleRange":
-                node = None
+            node = None
         return node
 
+    def get_super_character_style_range(self):
+        node = self.getparent()
+        if node.tag != "CharacterStyleRange":
+            node = None
+        return node
+
+    # TODO: factorize with Story.get_element_content_nodes().
+    def get_element_content_nodes(self):
+        return self.xpath(("./ParagraphStyleRange/CharacterStyleRange/Content | "
+                           "./CharacterStyleRange/Content | "
+                           "./XMLElement/CharacterStyleRange/Content | "
+                           "./Content"))
+
     def to_xml_structure_element(self):
         """Return the node as seen in the Structure panel of InDesign. """
         attrs = dict(self.attrib)
