@@ -310,7 +310,12 @@ class IDMLPackage(zipfile.ZipFile):
             return parent.get(font_style_to_apply, font_style_to_apply)
 
         def _get_nested_style_range_node(xml_structure_node):
-            """Use the more distant parent as base style and then apply its children styles until the new tag itself."""
+            """Use the more distant parent as base style and then apply its children styles until the new tag itself.
+
+            Returns:
+             o new_style_range_node: unbound element.
+             o root_style_node
+            """
             nested_styles = []
             while(xml_structure_node is not None):
                 style_name = self.style_mapping.character_style_mapping.get(xml_structure_node.tag)
@@ -425,6 +430,7 @@ class IDMLPackage(zipfile.ZipFile):
             "content": ["foo", {subtree}, "bar", ...]
         }
         """
+
         def _export_content_as_tree(xml_structure_node):
             content = []
             tree = {"tag": xml_structure_node.tag,
@@ -508,7 +514,7 @@ class IDMLPackage(zipfile.ZipFile):
             os.rename(old_name, new_name)
 
         # Update designmap.xml.
-        self.designmap.prefix_page_start(prefix)
+        self.designmap.prefix(prefix)
         self.designmap.synchronize()
 
         return self
@@ -887,6 +893,10 @@ class IDMLPackage(zipfile.ZipFile):
 
         return spread_elements
 
+    def get_spread_object_by_name(self, name):
+        """ - name : 'Spreads/Spread_mainub6.xml' """
+        return filter(lambda s: s.name == name, self.spreads_objects)[0]
+
     def get_spread_object_by_xpath(self, xpath):
         elt_id = self.xml_structure.xpath(xpath)[0].get("XMLContent")
         return self.get_spread_object_by_id(elt_id)
@@ -932,6 +942,9 @@ class IDMLPackage(zipfile.ZipFile):
     def get_layer_id_by_name(self, layer_name):
         return self.designmap.get_layer_id_by_name(layer_name)
 
+    def get_active_layer_name(self):
+        return self.designmap.get_active_layer_name()
+
     def get_structure_element_layer_id(self, xml_element):
         spread_elt = self.get_spread_elem_by_id(xml_element.get("XMLContent"))
         return self.get_spread_element_layer_id(spread_elt)
