@@ -14,8 +14,37 @@ from tempfile import mkdtemp
 CURRENT_DIR = os.path.abspath(os.path.split(__file__)[0])
 SCRIPTS_DIR = os.path.join(CURRENT_DIR, "scripts")
 
+JS_SAVE_AS_SCRIPT = "save_as.jsx"
+JS_EXPORT_SCRIPT = "export.jsx"
+JS_PACKAGE_SCRIPT = "package_to_print.jsx"
+JS_CLOSE_ALL_SCRIPT = "close_all_documents.jsx"
+JS_SAVE_AS_SCRIPTS = [JS_SAVE_AS_SCRIPT, JS_EXPORT_SCRIPT, JS_PACKAGE_SCRIPT]
 
-def save_as(src_filename, dst_formats, indesign_server_url, indesign_client_workdir,
+
+def close_all_documents(indesign_server_url, indesign_client_workdir, indesign_server_workdir,
+                        indesign_server_path_style="posix", ftp_params=None):
+    server_path_mod = os.path
+    if indesign_server_path_style == "windows":
+        server_path_mod = ntpath
+
+    javascript_basename = JS_CLOSE_ALL_SCRIPT
+    javascript_master_filename = os.path.join(SCRIPTS_DIR, javascript_basename)
+    javascript_client_copy_filename = os.path.join(indesign_client_workdir, javascript_basename)
+    javascript_server_copy_filename = server_path_mod.join(indesign_server_workdir, javascript_basename)
+
+    _copy(javascript_master_filename, javascript_client_copy_filename, ftp_params, src_open_mode="r")
+
+    cl = Client("%s/service?wsdl" % indesign_server_url)
+    cl.set_options(location=indesign_server_url)
+
+    params = cl.factory.create("ns0:RunScriptParameters")
+    params.scriptLanguage = 'javascript'
+    params.scriptFile = javascript_server_copy_filename
+
+    cl.service.RunScript(params)
+
+
+def save_as(src_filename, dst_formats_params, indesign_server_url, indesign_client_workdir,
             indesign_server_workdir, indesign_server_path_style="posix",
             clean_workdir=True, ftp_params=None):
     """SOAP call to an InDesign Server to make one or more conversions. """
@@ -24,18 +53,21 @@ def save_as(src_filename, dst_formats, indesign_server_url, indesign_client_work
     if indesign_server_path_style == "windows":
         server_path_mod = ntpath
 
-    def _save_as(dst_format):
+    def _save_as(dst_format_params):
         """
         o *_client_copy_filename : path/to/file as seen by the SOAP client.
         o *_server_copy_filename : localized/path/to/file as seen by the InDesign Server.
         """
+        dst_format = dst_format_params["fmt"]
+        js_params = dst_format_params.get("params", {})
+
         src_rootname = os.path.splitext(src_basename)[0]
-        dst_basename = "%s.%s" % (src_rootname, dst_format)
-        javascript_basename = "save_as.jsx"
+        dst_basename = "%sTMP.%s" % (src_rootname, dst_format)
+        javascript_basename = JS_SAVE_AS_SCRIPT
         if dst_format in ('idml', 'pdf', 'jpeg'):
-            javascript_basename = "export.jsx"
+            javascript_basename = JS_EXPORT_SCRIPT
         elif dst_format == 'zip':
-            javascript_basename = "package_to_print.jsx"
+            javascript_basename = JS_PACKAGE_SCRIPT
             dst_basename = src_rootname  # a directory.
 
         javascript_master_filename = os.path.join(SCRIPTS_DIR, javascript_basename)
@@ -45,7 +77,7 @@ def save_as(src_filename, dst_formats, indesign_server_url, indesign_client_work
         javascript_server_copy_filename = server_path_mod.join(indesign_server_workdir, javascript_basename)
         response_server_copy_filename = server_path_mod.join(indesign_server_workdir, dst_basename)
 
-        _copy(javascript_master_filename, javascript_client_copy_filename, ftp_params)
+        _copy(javascript_master_filename, javascript_client_copy_filename, ftp_params, src_open_mode="r")
 
         params = cl.factory.create("ns0:RunScriptParameters")
         params.scriptLanguage = 'javascript'
@@ -61,6 +93,15 @@ def save_as(src_filename, dst_formats, indesign_server_url, indesign_client_work
 
         params.scriptArgs = [src, dst]
 
+        # Extra parameters
+        extra_params = []
+        for k, v in js_params.items():
+            param = cl.factory.create("ns0:IDSP-ScriptArg")
+            param.name = k
+            param.value = v
+            extra_params.append(param)
+        params.scriptArgs.extend(extra_params)
+
         if dst_format in ('idml', 'pdf', 'jpeg'):
             fmt = cl.factory.create("ns0:IDSP-ScriptArg")
             fmt.name = "format"
@@ -93,7 +134,7 @@ def save_as(src_filename, dst_formats, indesign_server_url, indesign_client_work
 
     cl = Client("%s/service?wsdl" % indesign_server_url)
     cl.set_options(location=indesign_server_url)
-    responses = map(lambda fmt: _save_as(fmt), dst_formats)
+    responses = map(lambda fmt: _save_as(fmt), dst_formats_params)
 
     if clean_workdir:
         _unlink(src_client_copy_filename, ftp_params)
@@ -101,15 +142,20 @@ def save_as(src_filename, dst_formats, indesign_server_url, indesign_client_work
     return responses
 
 
-def _copy(src_filename, dst_filename, ftp_params=None):
+def _copy(src_filename, dst_filename, ftp_params=None, src_open_mode="rb"):
     if not ftp_params:
         shutil.copy(src_filename, dst_filename)
         return
-    with open(src_filename, "rb") as f:
+
+    with open(src_filename, src_open_mode) as f:
         ftp = ftplib.FTP(*ftp_params["auth"])
         ftp.set_pasv(ftp_params["passive"])
+        command = 'STOR %s' % dst_filename
         try:
-            ftp.storbinary('STOR %s' % dst_filename, f)
+            if "b" in src_open_mode:
+                ftp.storbinary(command, f)
+            else:
+                ftp.storlines(command, f)
         except BaseException, e:
             print "Cannot STOR %s" % dst_filename
             ftp.quit()
@@ -224,4 +270,3 @@ def rmtree_ftp(ftp, path):
         ftp.rmd(path)
     except ftplib.all_errors as e:
         raise e
-
